#ifdef _WIN32
#pragma warning (disable : 4786)
#endif

#include "qapplicationwindow.h"
#include "qrulebook.h"

#include <qimage.h>
#include <qpixmap.h>
#include <qtoolbar.h>
#include <qtoolbutton.h>
#include <qmenubar.h>
#include <qkeycode.h>
#include <qfile.h>
#include <qfiledialog.h>
#include <qmessagebox.h>
#include <qapplication.h>
#include <qaccel.h>

#ifdef _WIN32 // For windows : all elements are in the same directory.
#include "filesave.xpm"
#include "fileopen.xpm"

#else

#include "data/icons/filesave.xpm"
#include "data/icons/fileopen.xpm"
#endif

#include "qpixmapboard.h"
#include "qcommandline.h"
#include "gamemanager.h"

ApplicationWindow::ApplicationWindow()
  : QMainWindow( 0, "Project Space Hulk", WDestructiveClose )
{
  main = new QInterfaceImpl(this);
  themeEngine = main->bsv->pb->getThemeEngine();
  
  QPixmap openIcon, saveIcon;
  openIcon = QPixmap( fileopen );
  saveIcon = QPixmap( filesave );
  
  // Create the toolbar with two icons for open and save.
  // They are currently not working. So we delete them until version 1.4
  /*
  fileTools = new QToolBar( this, "file operations" );
  fileTools->setLabel( tr( "File Operations" ) );
  
  new QToolButton( openIcon, "Open File", QString::null,
		   this, SLOT(load()), fileTools, "open file" );

  new QToolButton( saveIcon, "Save File", QString::null,
		       this, SLOT(save()), fileTools, "save file" );
  */

  QPopupMenu * file = new QPopupMenu( this );
  menuBar()->insertItem( "&File", file );

  int id;
  file->insertItem( "&New Game", this, SLOT(newGame()), CTRL+Key_N );
  id = file->insertItem( openIcon, "&Load", this, SLOT(load()), CTRL+Key_L );
  file->setItemEnabled(id, false);
  id = file->insertItem( saveIcon, "&Save", this, SLOT(save()), CTRL+Key_S );
  file->setItemEnabled(id, false);
  id = file->insertItem( "Quick Save", this, SLOT(quicksave()), Key_F5 );
  file->setItemEnabled(id, false);
  id = file->insertItem( "Quick Load", this, SLOT(quickload()), Key_F6 );
  file->setItemEnabled(id, false);
  file->insertSeparator();
  file->insertItem( "&Quit", qApp, SLOT( closeAllWindows() ), CTRL+Key_Q );

  QPopupMenu * cmd = new QPopupMenu( this );
  menuBar()->insertItem( "&Command", cmd );
  cmd->insertItem( "Redo Previous Command", this, SLOT(redo()), Key_F1 );
  
  view = new QPopupMenu ( this );
  menuBar()->insertItem(" &View", view);
  view->insertItem("&Mission description", this, SLOT(printLevelInfo()),
		    CTRL+Key_M);
  buttonObjective = view->insertItem( "Objective squares in green", this,
				      SLOT(viewObjective()),Key_F10);
  view->insertItem( "Z&oom in/out", this, SLOT(switchview()), CTRL+Key_O );
  buttonRandomGround = view->insertItem( "Use random ground", this, SLOT(useRandomGround()) );
  view->setItemChecked(buttonRandomGround,true);

  view->insertSeparator();
  
  themes = new QPopupMenu (this);
  view->insertItem( "Themes", themes );
  set<string> *ths = themeEngine->getAllThemes();
  for (set<string>::iterator it=ths->begin(); it!=ths->end(); it++) {
    int id = themes->insertItem(it->c_str(), this, SLOT(changeTheme(int)));
    themes->setItemParameter(id, id);
    themesId[id] = *it;
    themesString[*it]=id;
  }
  themes->setItemChecked(themesString[themeEngine->getCurrentTheme()], true);
  
  menuBar()->insertSeparator();
  QPopupMenu * help = new QPopupMenu( this );
  menuBar()->insertItem( "&Help", help );

  help->insertItem( "&Rulebook", this, SLOT(rulebook()), CTRL+Key_H );
  help->insertSeparator();
  help->insertItem( "&About", this, SLOT(about()), Key_F12);
  help->insertItem( "About &Qt", this, SLOT(aboutQt()) );

  
  QAccel *acc = new QAccel( this );        // create accels for main window
  acc->connectItem( acc->insertItem(Key_Up), this, SLOT(boardUp()) );
  acc->connectItem( acc->insertItem(Key_Down), this, SLOT(boardDown()) );
  acc->connectItem( acc->insertItem(Key_Left), this, SLOT(boardLeft()) );
  acc->connectItem( acc->insertItem(Key_Right), this, SLOT(boardRight()) );


  setCentralWidget( main );

  resize( 800, 600 );

}


ApplicationWindow::~ApplicationWindow()
{}

QInterfaceImpl * ApplicationWindow::getMainWindow()
{ return main; }

void ApplicationWindow::newGame()
{
  main->cmd->getCommand("newgame");
}

void ApplicationWindow::load()
{
  QString fn = QFileDialog::getOpenFileName( QString::null, QString::null,
					     this);
  if ( !fn.isEmpty() )
    load( fn );
}

void ApplicationWindow::load( const char *fileName )
{
  QFile f( fileName );
  if ( !f.open( IO_ReadOnly ) )
    return;
    
}

void ApplicationWindow::quickload()
{
  if ( quickfile.isEmpty() ) {
    quickfile = QFileDialog::getOpenFileName(QString::null,QString::null,this);
    if (quickfile.isEmpty()) {
      return;
    }
  }

  load(quickfile);
}

void ApplicationWindow::quicksave()
{
  save("/home/romain/prog/spacehulk/src/essai.gz");
  //  if ( quickfile.isEmpty() ) {
  //  quickfile = QFileDialog::getSaveFileName(QString::null,QString::null,this);
  //  if (quickfile.isEmpty()) {
  //    return;
  //  }
  //}

  //save(quickfile);
}

void ApplicationWindow::save()
{
  QString fn = QFileDialog::getSaveFileName(QString::null, QString::null,
						  this );
  save(fn);
}

void ApplicationWindow::save(QString filename)
{
  // TODO : This should be getCommand action.
  man->Save(filename);
}

void ApplicationWindow::printLevelInfo(void)
{
  main->cmd->getCommand("view missiondescription");
}

void ApplicationWindow::switchview(void)
{
  main->switchView();
}

void ApplicationWindow::useRandomGround(void)
{
  if (view->isItemChecked(buttonRandomGround))
    view->setItemChecked(buttonRandomGround,false);
  else
    view->setItemChecked(buttonRandomGround,true);
  themeEngine->setRandomGround(view->isItemChecked(buttonRandomGround));
  main->cmd->DrawInterface();
}

void ApplicationWindow::viewObjective(void)
{
  if (view->isItemChecked(buttonObjective)) {
    view->setItemChecked(buttonObjective,false);
    main->bsv->pb->delGreenCases();
  }
  else {
    view->setItemChecked(buttonObjective,true);
    main->bsv->pb->setGreenCases(man->getObjectiveCases());
    man->printMissionObjectives();
  }
  main->cmd->DrawInterface();
}

void ApplicationWindow::changeTheme(int themeId)
{
  themes->setItemChecked(themesString[themeEngine->getCurrentTheme()], false);
  themeEngine->setTheme(themesId[themeId]);  
  themes->setItemChecked(themesString[themeEngine->getCurrentTheme()], true);
  main->cmd->DrawInterface();
}

void ApplicationWindow::closeEvent( QCloseEvent* ce )
{
  /* TODO : We will not check if game has been save. Later...
  if ( !e->edited() ) {
    ce->accept();
    return;
  }

  switch( QMessageBox::information( this, "Qt Application Example",
				    "The document has been changed since "
				    "the last save.",
				    "Save Now", "Cancel", "Leave Anyway",
				    0, 1 ) ) {
  case 0:
    save();
    ce->accept();
    break;
  case 1:
  default: // just for sanity
    ce->ignore();
    break;
  case 2:
    ce->accept();
    break;
    } 
  */
  main->cmd->endOfExecution();
  exit(0);
  ce->accept();
}

void ApplicationWindow::rulebook()
{
  QRuleBook rule(this);
  rule.exec();
}

void ApplicationWindow::about()
{
  QMessageBox::about( this, "QSpaceHulk",
		      "Conversion of the spacehulk board game.\n"
		      "Web site :\n"
		      "http://savannah.nongnu.org/projects/spacehulk\n"
		      "Main Author: Romain Vinot.\n");
}
    
void ApplicationWindow::aboutQt()
{
  QMessageBox::aboutQt( this, "Qt is a multi-platform GUI library." );
}

void ApplicationWindow::redo()
{
  string cmd = main->cmd->getPreviousCommand();
  main->cmd->getCommand(cmd.c_str());
}

void ApplicationWindow::boardUp() { main->bsv->boardUp(); }
void ApplicationWindow::boardDown() { main->bsv->boardDown(); }
void ApplicationWindow::boardLeft() { main->bsv->boardLeft(); }
void ApplicationWindow::boardRight() { main->bsv->boardRight(); }
