/***************************************************************************
                          qcommandline.cpp  -  description
                             -------------------
    begin                : ven avr 19 13:21:00 CET 2002
    copyright            : (C) 2002 by Romain Vinot
    email                : vinot@aist.enst.fr
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifdef _WIN32
#pragma warning (disable : 4786)
#endif

#include <cstdio> // For sscanf
#include <qmultilinedit.h>
#include <qlineedit.h>
#include <qlabel.h>
#include <qpushbutton.h>
#include <qmessagebox.h>
#include <qfiledialog.h>
#include <qstring.h>
#include <qthread.h>

#include "qcommandline.h"

#include "qinterfaceimpl.h"
#include "qmyevent.h"
#include "qdirectiondialogimpl.h"
#include "qlevelinfoimpl.h"
#include "qcreature.h"
#include "qpixmapboard.h"
#include "gamemanager.h"

QCommandLine::QCommandLine(QPixmapBoard *x, QCreature *c, QInterfaceImpl *main)
  : GUICommandLine(x,c), QThread(),  
    qmain(main),
    qinfo(main->qinfo), qcmd(main->qcommandline),
    qview(main->qViewLabel), qsel(main->qSelectedLabel),
    qlist(main->qListCreatures), qViewButton(main->qViewButton),
    qturninfo(main->qTurnLabel), qstate(Q_NONE)
{
  QThread::start();
}

QCommandLine::~QCommandLine(void)
{}


void QCommandLine::endOfExecution(void)
{

  currCmd="exit";

  gameLock.wakeOne();

}


void QCommandLine::run()
{
  gameMutex.lock();
  while (1) {
    Game_Wait();

    GUICommandLine::getCommand(currCmd.c_str());
  }
}

// This method is executed by the GUI-thread.
void QCommandLine::Game_Wake(void)
{
  gameLock.wakeOne();

}

void QCommandLine::Game_Wait(void)
{
  gameLock.wait(&gameMutex);
  if (currCmd=="exit")

	  QThread::exit();

}

// This method is executed by the GUI-thread.
void QCommandLine::getCommand(const char *command)
{
  currCmd = string(command);
  Game_Wake();
}

void QCommandLine::GetEvent(void)
{
  switch(qstate) {
  case Q_DIRECTION: GUI_GetDirection(); break;
  case Q_LEVEL: GUI_GetLevelToPlay(); break;
  case Q_BOOLEAN: GUI_GetYesOrNo(); break;
  case Q_ENDOFGAME: GUI_EndOfGame(); break;
  case Q_LEVELINFO: GUI_PrintLevelInfo(); break;
  case Q_POPUPMSG: GUI_PrintPopupMessage(); break;
  default: break;
  }
}

void QCommandLine::Write(string w, bool isCommand)
{
  postEvent(qmain, new WriteCustomEvent(1000,w));
}

void QCommandLine::ResizeBoard(int x, int y)
{
  postEvent(qmain, new ResizeCustomEvent(x,y));
}

void QCommandLine::setObjectButtonsVisible(bool vis)
{
  postEvent(qmain, new ButtonShowHideCustomEvent(vis));
}

void QCommandLine::DrawInterface(void)
{
  postEvent(qmain, new DrawInterfaceCustomEvent(true));
  man->printAllCreaturesInfo();
  man->printSelectedCreature();
  man->printOnViewCreature();
}

void QCommandLine::GUIDrawInterface(void)
{
  qcmd->clear();
  qview->clear();
  qsel->clear();
  guiboard->printBoard();
  if (man->onViewIsSelectable())
    qViewButton->setEnabled(TRUE);
  else
    qViewButton->setEnabled(false);
  qturninfo->setText(man->getLevelMessage().c_str());
}

void QCommandLine::ClearListCreatures(void)
{
  postEvent(qmain, new DrawInterfaceCustomEvent(false));
}

void QCommandLine::GUIClearListCreatures(void)
{
  qlist->clear();
}

void QCommandLine::boardClick(int xcoord, int ycoord, bool isLeftButton)
{
  char com[64];
  if (isLeftButton) {
    sprintf(com, "%d %d", xcoord, ycoord);
    getCommand(com);
  }
  else {
    string tmp = man->chooseAction(xcoord,ycoord);
    getCommand(tmp.c_str());
  }
}

Direction QCommandLine::GetDirection(string txt)
{
  gui_dir=NODIRECTION;
  gui_msg = txt;
  qstate = Q_DIRECTION;
  postEvent(qmain, new QCustomEvent(346798));
  Game_Wait(); // The GUI-thread will put the info on the gui_dir variable.
  return gui_dir;
}

void QCommandLine::GUI_GetDirection(void)
{
  QDirectionDialogImpl ask(&gui_dir);
  ask.setLabel(gui_msg);
  ask.exec();
  qstate = Q_NONE;
  Game_Wake();
}

QString QCommandLine::GetLevelToPlay(void)
{
  gui_level="";
  qstate = Q_LEVEL;
  postEvent(qmain, new QCustomEvent(346798));
  Game_Wait(); // The GUI-thread will put the info on the gui_level variable.
  return gui_level;
}

void QCommandLine::GUI_GetLevelToPlay(void)
{
  char path[1024];
  sprintf(path,"%s%clevels",DATA_DIR, DIR_DELIMITER);
  QString s( QFileDialog::getOpenFileName( path, "Mission file (*.lvl)") );
  if ( s.isEmpty() )
    return;
  
  gui_level = s;
  qstate = Q_NONE;
  Game_Wake();
}

int QCommandLine::GetCase(set<int> *cases, string txt)
{
  int caseId = -1;
  Write(txt);
  guiboard->setYellowCases(cases);
  DrawInterface();
  while (1) {
    Game_Wait();
    if (!getMainCommands(currCmd.c_str())) {
      int xcoord, ycoord;
      int nb = sscanf(currCmd.c_str(),"%d %d", &xcoord, &ycoord);
      caseId = xcoord+ycoord*board->Xmax;
      if (nb!=2) {
	Write("You must choose a yellow square. No other command is allowed.");
      }
      else if (cases->find(caseId)!=cases->end())
	break;
      else {
	string msg ="This square is not allowed. Please choose another one.\n";
	msg += "Only yellow squares are allowed.";
	Write(msg);
      }
    }
  }
  guiboard->delYellowCases();
  return caseId;
}

pair<int,Direction> QCommandLine::GetCaseAndDirection(set<int> *ca, string txt)
{
  int caseId = GetCase(ca, txt);
  Direction dir = GetDirection("Give an orientation.\n");
  return pair<int, Direction>(caseId, dir);
}

bool QCommandLine::GetYesOrNo(string txt)
{
  gui_res=false;
  gui_msg = txt;
  qstate = Q_BOOLEAN;
  postEvent(qmain, new QCustomEvent(346798));

  Game_Wait(); // The GUI-thread will put the info on the gui_dir variable.
  return gui_res;
}

void QCommandLine::GUI_GetYesOrNo(void)
{
  int res=QMessageBox::information(qmain, "QSpacehulk - Answer the question", 
			   gui_msg.c_str(), "Yes", "No");
  gui_res = (res==0);
  qstate = Q_NONE;
  Game_Wake();
}

void QCommandLine::endOfGame(string txt)
{
  gui_msg = txt;
  qstate = Q_ENDOFGAME;
  postEvent(qmain, new QCustomEvent(346798));
  Game_Wait();
}

void QCommandLine::GUI_EndOfGame(void)
{
  QMessageBox::about( qmain, "QSpaceHulk - End Of Game",
		      gui_msg.c_str());
  qstate = Q_NONE;
  Game_Wake();
}

void QCommandLine::printLevelInfo(QString &title, QString &auth, QString &nar,
				  QString &obj, QString &forc, QString &cond)
{
  leveltitle = title;
  levelauthor = auth;
  levelnarration = nar;
  levelobjectives = obj;
  levelforces = forc;
  levelcond = cond;
  qstate = Q_LEVELINFO;
  postEvent(qmain, new QCustomEvent(346798));
  Game_Wait();
}

void QCommandLine::printLevelInfo(void)
{
  man->getLevelInfo(leveltitle, levelauthor, levelnarration, levelobjectives,
		    levelforces, levelcond);
  qstate = Q_LEVELINFO;
  postEvent(qmain, new QCustomEvent(346798));
  Game_Wait();
}

void QCommandLine::GUI_PrintLevelInfo(void)
{
  QLevelInfoImpl lev(qmain, leveltitle, levelauthor, levelnarration,
		     levelobjectives, levelforces, levelcond);
  lev.exec();
  qstate = Q_NONE;
  Game_Wake();
}

void QCommandLine::printPopupMessage(string msg)
{
  gui_msg = msg;
  qstate = Q_POPUPMSG;
  postEvent(qmain, new QCustomEvent(346798));
  Game_Wait();
}

void QCommandLine::GUI_PrintPopupMessage(void)
{
  QMessageBox::about( qmain, "QSpaceHulk - Message",
		      gui_msg.c_str());
  qstate = Q_NONE;
  Game_Wake();
}
